#include <stdlib.h>
#include "anamod.h"
#include "anamodsalsamodules.h"

#undef __FUNCT__
#define __FUNCT__ "AnaModRegisterSalsaModules"
PetscErrorCode AnaModRegisterSalsaModules()
{
  PetscErrorCode ierr;
  PetscFunctionBegin;

  ierr = RegisterSimpleModules(); CHKERRQ(ierr);
  ierr = RegisterVarianceModules(); CHKERRQ(ierr);
  ierr = RegisterNormalityModules(); CHKERRQ(ierr);
  ierr = CommutatorNormFAllowSqrtTimes(10);
  ierr = RegisterStructureModules(); CHKERRQ(ierr);
  ierr = RegisterSpectrumModules(); CHKERRQ(ierr);
  ierr = SpectrumComputeUnpreconditionedSpectrum(); CHKERRQ(ierr);
  ierr = RegisterJPLModules(); CHKERRQ(ierr);
  ierr = RegisterIprsModules(); CHKERRQ(ierr);
#if defined(HAVE_LAPACK)
  ierr = RegisterLapackModules(); CHKERRQ(ierr);
#endif

  PetscFunctionReturn(0);
}

#undef __FUNCT__
#define __FUNCT__ "AnaModDeregisterSalsaModules"
PetscErrorCode AnaModDeregisterSalsaModules()
{
  PetscErrorCode ierr;
  PetscFunctionBegin;

  ierr = DeRegisterSimpleModules(); CHKERRQ(ierr);
  ierr = DeRegisterVarianceModules(); CHKERRQ(ierr);
  ierr = DeRegisterStructureModules(); CHKERRQ(ierr);
  ierr = DeregisterSpectrumModules(); CHKERRQ(ierr);

  PetscFunctionReturn(0);
}

#undef __FUNCT__
#define __FUNCT__ "AnaModRegisterStandardModules"
/*! Register all standard and nonstandard analysis modules.
*/
PetscErrorCode AnaModRegisterStandardModules()
{
  PetscErrorCode ierr;
  PetscFunctionBegin;

  /* standard set */
  ierr = RegisterSimpleModules(); CHKERRQ(ierr);
  ierr = RegisterVarianceModules(); CHKERRQ(ierr);
  ierr = RegisterStructureModules(); CHKERRQ(ierr);
  ierr = RegisterSpectrumModules(); CHKERRQ(ierr);
  ierr = RegisterNormalityModules(); CHKERRQ(ierr);
  CHKMEMQ;

  /* UKY iprs set */
  ierr = RegisterIprsModules(); CHKERRQ(ierr);

  /* Eijkhout's ICMK */
  /*ierr = RegisterICMKModules(); CHKERRQ(ierr);*/

#if defined(HAVE_LAPACK)
  /* Lapack eigenvalue calculations */
  ierr = RegisterLapackModules(); CHKERRQ(ierr);
#endif

  /* Jones/Plassmann multi-colouring */
  ierr = RegisterJPLModules(); CHKERRQ(ierr);
  CHKMEMQ;

  PetscFunctionReturn(0);
}

