#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "nmd_impl.h"
#include "nmd.h"

#if defined(NMD_HAVE_HDF5)
#include "hdf5.h"
#define H5ERR(i) if (i<0) {NMD_ERR_RETURN(i);}

/*! Generate an HDF5 dump of an object
 */
#undef __FUNCT__
#define __FUNCT__ "NMDObjectDumpToHDF5"
NMDErrorCode NMDObjectDumpToHDF5(NMD_metadata obj,const char *fname)
{
  hid_t f,element,ds; hsize_t dim[1]; herr_t status;
  int icat,icmp;
  NMDErrorCode ierr;
  CHECKHASNMDCOOKIE(obj);

  f = H5Fcreate
    (fname,H5F_ACC_TRUNC,H5P_DEFAULT,H5P_DEFAULT); H5ERR(f);
  dim[0] = 1;
  element = H5Screate_simple(1,dim,NULL); H5ERR(element);

  for (icat=0; icat<obj->ncat; icat++) {
    NMD_metadata_category cat = obj->cats[icat];
    NMD_string catstring,grpstring; const char *c;
    CHECKHASNMDCOOKIE(cat);
    ierr = NMDStringCreate(cat->name,&catstring); NMD_ERR_RETURN(ierr);
    ierr = NMDStringConcat
      ('/',catstring,0,0,0,&grpstring); NMD_ERR_RETURN(ierr);
    ierr = NMDStringGetString(grpstring,&c); NMD_ERR_RETURN(ierr);
    ds = H5Gcreate(f,c,0,H5P_DEFAULT,H5P_DEFAULT); H5ERR(ds);

    for (icmp=0; icmp<cat->ncmp; icmp++) {
      NMD_metadata_item cmp = cat->cmps[icmp];
      NMD_string cmpstring,cmpcatstring;
      CHECKHASNMDCOOKIE(cmp);
      ierr = NMDStringCreate(cmp->name,&cmpstring); NMD_ERR_RETURN(ierr);
      ierr = NMDStringConcat
	('/',catstring,'/',cmpstring,0,&cmpcatstring); NMD_ERR_RETURN(ierr);
      ierr = NMDStringGetString(cmpcatstring,&c); NMD_ERR_RETURN(ierr);
#define HDF5DOUBLE H5T_NATIVE_DOUBLE
#define HDF5INT H5T_NATIVE_INT
      //#define HDF5INT H5T_STD_I32LE
      switch (cmp->t) {
      case NMDInt :
	ds = H5Dcreate (f,c,HDF5INT,element,H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT); H5ERR(ds);
	status = H5Dwrite
	  (ds, HDF5INT,H5S_ALL,H5S_ALL,H5P_DEFAULT,
	   &(cmp->i)); H5ERR(status);
	break ;
      case NMDReal :
	ds = H5Dcreate (f,c,HDF5DOUBLE,element,H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT); H5ERR(ds);
	status = H5Dwrite
	  (ds, HDF5DOUBLE,H5S_ALL,H5S_ALL,H5P_DEFAULT,
	   &(cmp->r)); H5ERR(status);
	break;
      case NMDIntarray :
	{
	  hid_t dataspace,memspace;
	  dim[0] = cmp->ii->length;
	  dataspace = H5Screate_simple(1,dim,NULL); H5ERR(dataspace);
	  memspace  = H5Screate_simple(1,dim,NULL); H5ERR(dataspace);
	  ds = H5Dcreate (f,c,HDF5INT,dataspace,H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT); H5ERR(ds);
	  status = H5Dwrite /* memspace,dataspace */
	    (ds, HDF5INT,memspace,dataspace,H5P_DEFAULT,
	     cmp->ii->data); H5ERR(status);
	  H5Sclose(memspace); H5Sclose(dataspace);
	  break ;
	}
      case NMDRealarray :
	{
	  hid_t dataspace,memspace;
	  dim[0] = cmp->rr->length;
	  dataspace = H5Screate_simple(1,dim,NULL); H5ERR(dataspace);
	  memspace  = H5Screate_simple(1,dim,NULL); H5ERR(dataspace);
	  ds = H5Dcreate (f,c,HDF5DOUBLE,dataspace,H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT); H5ERR(ds);
	  status = H5Dwrite /*  */
	    (ds, HDF5DOUBLE,memspace,dataspace,H5P_DEFAULT,
	     cmp->rr->data); H5ERR(status);
	  H5Sclose(memspace); H5Sclose(dataspace);
	  break ;
	}
      default :
	break ;
      }
      ierr = NMDStringDestroy(cmpstring); NMD_ERR_RETURN(ierr);
      ierr = NMDStringDestroy(cmpcatstring); NMD_ERR_RETURN(ierr);
    }
    ierr = NMDStringDestroy(catstring); NMD_ERR_RETURN(ierr);
    ierr = NMDStringDestroy(grpstring); NMD_ERR_RETURN(ierr);
  }
      
  status = H5Fclose(f);

  return 0;
}
#endif

